@@ -48,6 +48,7 @@
 #include <math.h>
 #include <limits.h>
 #include <string.h>
+#include <signal.h>
 #include <vector>
 #include <iterator>
 #include <numeric>
@@ -60,7 +61,7 @@
 #include <cstdint>
 #include <iostream>
 #ifndef _FOR_R
-    #include <cstdio> 
+    #include <stdio.h> 
 #else
     extern "C" {
         #include <R_ext/Print.h>
@@ -198,9 +199,10 @@ typedef struct IsoTree {
             this->remainder
             );
     }
-    IsoTree() {};
     #endif
 
+    IsoTree() = default;
+
 } IsoTree;
 
 typedef struct IsoHPlane {
@@ -243,8 +245,9 @@ typedef struct IsoHPlane {
             this->remainder
             );
     }
-    IsoHPlane() {};
     #endif
+
+    IsoHPlane() = default;
 } IsoHPlane;
 
 /* Note: don't use long doubles in the outside outputs or there will be issues with MINGW in windows */
@@ -273,8 +276,9 @@ typedef struct IsoForest {
             this->orig_sample_size
             );
     }
-    IsoForest() {};
     #endif
+
+    IsoForest() = default;
 } IsoForest;
 
 typedef struct ExtIsoForest {
@@ -300,8 +304,9 @@ typedef struct ExtIsoForest {
             this->orig_sample_size
             );
     }
-    ExtIsoForest() {};
     #endif
+
+    ExtIsoForest() = default;
 } ExtIsoForest;
 
 typedef struct ImputeNode {
@@ -324,7 +329,7 @@ typedef struct ImputeNode {
             );
     }
     #endif
-    ImputeNode() {};
+    ImputeNode() = default;
 
     ImputeNode(size_t parent)
     {
@@ -356,7 +361,7 @@ typedef struct Imputer {
     }
     #endif
 
-    Imputer() {};
+    Imputer() = default;
 
 } Imputer;
 
@@ -418,6 +423,7 @@ typedef struct {
     size_t ndim;        /* only for extended model */
     size_t ntry;        /* only for extended model */
     CoefType coef_type; /* only for extended model */
+    bool coef_by_prop;  /* only for extended model */
 
     bool calc_dist;     /* checkbox for calculating distances on-the-fly */
     bool calc_depth;    /* checkbox for calculating depths on-the-fly */
@@ -527,6 +533,8 @@ typedef struct WorkerForSimilarity {
     std::vector<double> weights_arr;
     std::vector<double> comb_val;
     std::vector<double> tmat_sep;
+    std::vector<double> rmat;
+    size_t              n_from;
     bool                assume_full_distr; /* doesn't need to have one copy per worker */
 } WorkerForSimilarity;
 
@@ -537,20 +545,20 @@ typedef struct {
     size_t  split_ix;
     size_t  end;
     std::vector<size_t> ix_arr;
-    std::unordered_map<size_t, double> weights_map;
-    std::vector<double> weights_arr;
     std::vector<bool>   cols_possible;
+    std::unique_ptr<double[]> weights_arr;
     std::discrete_distribution<size_t> col_sampler;
 } RecursionState;
 
 /* Function prototypes */
 
 /* fit_model.cpp */
+extern bool interrupt_switch;
 int fit_iforest(IsoForest *model_outputs, ExtIsoForest *model_outputs_ext,
                 double numeric_data[],  size_t ncols_numeric,
                 int    categ_data[],    size_t ncols_categ,    int ncat[],
                 double Xc[], sparse_ix Xc_ind[], sparse_ix Xc_indptr[],
-                size_t ndim, size_t ntry, CoefType coef_type,
+                size_t ndim, size_t ntry, CoefType coef_type, bool coef_by_prop,
                 double sample_weights[], bool with_replacement, bool weight_as_sample,
                 size_t nrows, size_t sample_size, size_t ntrees, size_t max_depth,
                 bool   limit_depth, bool penalize_range,
@@ -568,7 +576,7 @@ int add_tree(IsoForest *model_outputs, ExtIsoForest *model_outputs_ext,
              double numeric_data[],  size_t ncols_numeric,
              int    categ_data[],    size_t ncols_categ,    int ncat[],
              double Xc[], sparse_ix Xc_ind[], sparse_ix Xc_indptr[],
-             size_t ndim, size_t ntry, CoefType coef_type,
+             size_t ndim, size_t ntry, CoefType coef_type, bool coef_by_prop,
              double sample_weights[], size_t nrows, size_t max_depth,
              bool   limit_depth,   bool penalize_range,
              double col_weights[], bool weigh_by_kurt,
@@ -653,7 +661,8 @@ void get_num_nodes(ExtIsoForest &model_outputs, sparse_ix *restrict n_nodes, spa
 void calc_similarity(double numeric_data[], int categ_data[],
                      double Xc[], sparse_ix Xc_ind[], sparse_ix Xc_indptr[],
                      size_t nrows, int nthreads, bool assume_full_distr, bool standardize_dist,
-                     IsoForest *model_outputs, ExtIsoForest *model_outputs_ext, double tmat[]);
+                     IsoForest *model_outputs, ExtIsoForest *model_outputs_ext,
+                     double tmat[], double rmat[], size_t n_from);
 void traverse_tree_sim(WorkerForSimilarity   &workspace,
                        PredictionData        &prediction_data,
                        IsoForest             &model_outputs,
@@ -668,12 +677,14 @@ void gather_sim_result(std::vector<WorkerForSimilarity> *worker_memory,
                        std::vector<WorkerMemory> *worker_memory_m,
                        PredictionData *prediction_data, InputData *input_data,
                        IsoForest *model_outputs, ExtIsoForest *model_outputs_ext,
-                       double *restrict tmat, size_t ntrees, bool assume_full_distr,
+                       double *restrict tmat, double *restrict rmat, size_t n_from,
+                       size_t ntrees, bool assume_full_distr,
                        bool standardize_dist, int nthreads);
 void initialize_worker_for_sim(WorkerForSimilarity  &workspace,
                                PredictionData       &prediction_data,
                                IsoForest            *model_outputs,
                                ExtIsoForest         *model_outputs_ext,
+                               size_t                n_from,
                                bool                  assume_full_distr);
 
 /* impute.cpp */
@@ -767,6 +778,10 @@ void increase_comb_counter(size_t ix_arr[], size_t st, size_t end, size_t n,
                            double *restrict counter, double *restrict weights, double exp_remainder);
 void increase_comb_counter(size_t ix_arr[], size_t st, size_t end, size_t n,
                            double counter[], std::unordered_map<size_t, double> &weights, double exp_remainder);
+void increase_comb_counter_in_groups(size_t ix_arr[], size_t st, size_t end, size_t split_ix, size_t n,
+                                     double counter[], double exp_remainder);
+void increase_comb_counter_in_groups(size_t ix_arr[], size_t st, size_t end, size_t split_ix, size_t n,
+                                     double *restrict counter, double *restrict weights, double exp_remainder);
 void tmat_to_dense(double *restrict tmat, double *restrict dmat, size_t n, bool diag_to_one);
 double calc_sd_raw(size_t cnt, long double sum, long double sum_sq);
 long double calc_sd_raw_l(size_t cnt, long double sum, long double sum_sq);
@@ -802,6 +817,9 @@ void get_categs(size_t ix_arr[], int x[], size_t st, size_t end, int ncat,
                 MissingAction missing_action, char categs[], size_t &npresent, bool &unsplittable);
 long double calculate_sum_weights(std::vector<size_t> &ix_arr, size_t st, size_t end, size_t curr_depth,
                                   std::vector<double> &weights_arr, std::unordered_map<size_t, double> &weights_map);
+void set_interrup_global_variable(int s);
+int return_EXIT_SUCCESS();
+int return_EXIT_FAILURE();
 
 
 
